package framework.services.defaultServices;

import framework.integration.simulation.Breakpoint;
import model.conection.HttpResponse;
import model.conection.Request;
import model.conection.Response;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import static exception.ExceptionHandler.getSafe;
import static framework.serverState.ServerState.state;

public class BreakpointService implements framework.services.avaliableServices.BreakpointService {

    public BreakpointService() {
    }

    @Override
    public Response create(Request request) {
        return createResponse(createBreakpoint(getDate(request)));
    }

    private Response createResponse(String breakpointId) {
        return new HttpResponse(200, "{\"message\":\"breakpoint created\",\"breakpointId\":\""+breakpointId+"\"}");
    }

    private Date getDate(Request request) {
        DateFormat format = new SimpleDateFormat("MM/dd/yyyy HH:mm:ss:SSS");
        DateFormat formatWithMilliseconds = new SimpleDateFormat("MM/dd/yyyy HH:mm:ss");
        return getSafe(() -> format.parse(request.getParameter("time")),
                ()-> getSafe(() -> formatWithMilliseconds.parse(request.getParameter("time"))));
    }

    @Override
    public Response delete(Request request) {
        return deleteResponse(delete(Integer.parseInt(request.getParameter("breakpointId"))));
    }


    private Response deleteResponse(boolean actionResult) {
        String message = "{\"message\":\""+ (actionResult ? "Breakpoint Deleted" : "Can't remove the breakpoint") +"\"}";
        return new HttpResponse(200, message);
    }

    public String createBreakpoint(Date date) {
        Breakpoint breakpoint = Breakpoint.create(date);
        state().breakpoints().add(breakpoint);
        return String.valueOf(breakpoint.id());
    }

    public boolean delete(int breakpointId) {
        state().breakpoints().removeById(breakpointId);
        return true;
    }
}
