package framework.services;

import framework.mockServices.classToService.BreakpointHandlerService;
import framework.mockServices.classToService.NotificationHandlerService;
import framework.mockServices.classToService.SensorHandlerService;
import framework.mockServices.classToService.SubscribeHandlerService;
import framework.services.avaliableServices.*;
import framework.services.servicesFinder.ReflectionServiceFinder;
import framework.services.servicesFinder.ServiceFinder;
import model.conection.HttpResponse;
import model.conection.Request;
import model.conection.Response;
import org.junit.Test;

import java.util.HashMap;
import java.util.Map;

import static junit.framework.Assert.assertSame;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

public class ServiceManagerTest {
    @Test
    public void shouldReturnABreakpointService() {
        ServiceFinder finder = mock(ServiceFinder.class);
        when(finder.search("framework")).thenReturn(getServiceList(BreakpointHandlerService.class));
        ServicesManager.setUp(finder, "framework");
        assertSame(ServicesManager.get(BreakpointService.class).getClass(), BreakpointHandlerService.class);
    }

    @Test
    public void shouldReturnAServiceBreakpoint() {
        ServiceFinder finder = mock(ServiceFinder.class);
        when(finder.search("framework")).thenReturn(getServiceList(BreakpointHandlerService.class));
        ServicesManager.setUp(finder, "framework");
        assertSame(ServicesManager.get(BreakpointService.class).getClass(), BreakpointHandlerService.class);
    }

    @Test
    public void shouldHasAllReferencesToUltraFakeClass() {
        ServiceFinder finder = mock(ServiceFinder.class);
        when(finder.search("framework")).thenReturn(getServiceList(AllServicesImplementation.class));
        ServicesManager.setUp(finder, "framework");
        assertSame(AllServicesImplementation.class, ServicesManager.get(BreakpointService.class).getClass());
        assertSame(AllServicesImplementation.class, ServicesManager.get(NotificationService.class).getClass());
        assertSame(AllServicesImplementation.class, ServicesManager.get(SensorService.class).getClass());
        assertSame(AllServicesImplementation.class, ServicesManager.get(SubscriptionService.class).getClass());
    }

    @Test
    public void integrationWithReflectionServiceFinder() {
        ServicesManager.setUp(new ReflectionServiceFinder(), "framework.mockServices.classToService");
        assertSame(BreakpointHandlerService.class, ServicesManager.get(BreakpointService.class).getClass());
        assertSame(SubscribeHandlerService.class, ServicesManager.get(SubscriptionService.class).getClass());
        assertSame(SensorHandlerService.class, ServicesManager.get(SensorService.class).getClass());
        assertSame(NotificationHandlerService.class, ServicesManager.get(NotificationService.class).getClass());
    }


    @Test
    public void testGetNotificationService() throws Exception {
        ServicesManager.setUp(new ReflectionServiceFinder(), "framework.mockServices.classToService");
        NotificationService notificationService = ServicesManager.get(NotificationService.class);
        assertSame(notificationService.getClass(), NotificationHandlerService.class);
    }

    private Map<Class<?>, Class<? extends FrameworkService>> getServiceList(Class<? extends FrameworkService> service) {
        Map<Class<?>, Class<? extends FrameworkService>> classes = new HashMap<>();
        for (Class<?> aClass : service.getInterfaces()) {
            classes.put(aClass, service);
        }
        return classes;
    }

    public static class AllServicesImplementation implements BreakpointService, SensorService, NotificationService, SubscriptionService{
        public AllServicesImplementation() {
        }

        @Override
        public Response create(Request breakpointId) {
            return null;
        }

        @Override
        public Response delete(Request request) {
            return null;
        }

        @Override
        public Response installSensor(Request request) {
            return null;
        }

        @Override
        public HttpResponse remove(Request request) {
            return null;
        }

        @Override
        public void push(String username, String message) {

        }

        @Override
        public void broadcast(String message) {

        }

        @Override
        public Response subscribe(Request request) {
            return null;
        }
    }
}
