package integrationTest;

import Controller.HttpPetitionClient;
import sites.Application;
import sites.route.identification.UserIdentification;
import sites.route.notification.NotificationServer;
import connection.NetInformation;
import framework.SGIFramework;
import framework.services.ServicesManager;
import framework.services.avaliableServices.NotificationService;
import junit.framework.TestCase;
import model.conection.HttpRequest;
import model.conection.Response;
import org.java_websocket.client.WebSocketClient;
import org.java_websocket.handshake.ServerHandshake;
import org.junit.BeforeClass;
import org.junit.Test;

import java.net.URI;
import java.net.URISyntaxException;

import static com.sun.javafx.fxml.expression.Expression.greaterThanOrEqualTo;
import static junit.framework.Assert.assertTrue;
import static junit.framework.TestCase.assertEquals;
import static junit.framework.TestCase.assertNotNull;
import static org.hamcrest.core.StringContains.containsString;
import static org.junit.Assert.assertThat;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;

public class IntegrationTest {

    public static final String ROOTS = "[{\"name\":\"happy-building\",\"objectId\":\"happy-building\"},{\"name\":\"bored-building\",\"objectId\":\"bored-building\"},{\"name\":\"unhappy-building\",\"objectId\":\"unhappy-building\"}]";
    private static SGIFramework simulation;

    @BeforeClass
    public static void setUp() throws Exception {
        Application.run(null);
        simulation = new integrationTest.Main.Service("simulation32", integrationTest.Main.initialDate(), integrationTest.Main.finalDate());
        simulation.waitForRequest(false);
    }



    @Test
    public void testServerDiscoverClient() throws Exception {
        NetInformation simulation32 = sites.ServerState.instance().getSubscription("simulation32");

        assertEquals("simulation32", simulation32.getHostName());
        assertEquals(50000, simulation32.getPort());
        greaterThanOrEqualTo(sites.ServerState.instance().numberOfSubscriptions(), 1);
        assertEquals(simulation32.getRemoteIP(), framework.serverState.ServerState.state().getRemoteAddress());
    }

    @Test
    public void testServerDiscoverALotOfClients() throws Exception {

        new integrationTest.Main.Service("simulation33", integrationTest.Main.initialDate(), integrationTest.Main.finalDate());
        new integrationTest.Main.Service("simulation34", integrationTest.Main.initialDate(), integrationTest.Main.finalDate());

        assertNotNull(sites.ServerState.instance().getSubscription("simulation32"));
        assertNotNull(sites.ServerState.instance().getSubscription("simulation33"));
        assertNotNull(sites.ServerState.instance().getSubscription("simulation34"));
        TestCase.assertTrue(sites.ServerState.instance().getSubscription("simulation34").getPort() != sites.ServerState.instance().getSubscription("simulation33").getPort());
        TestCase.assertTrue(sites.ServerState.instance().getSubscription("simulation34").getPort() != sites.ServerState.instance().getSubscription("simulation32").getPort());
        TestCase.assertTrue(sites.ServerState.instance().getSubscription("simulation33").getPort() != sites.ServerState.instance().getSubscription("simulation32").getPort());
    }

    @Test
    public void getResponseFromCreateBreakpointEndpoint() throws Exception {
        Response response = new HttpPetitionClient().sendRequest(new HttpRequest("POST", framework.serverState.ServerState.state().getRemoteAddress() + ":8080/simulation32/Breakpoint", "{\"time\":\"tiempo\"}"));
        assertEquals(200, response.getStatusCode());
        assertThat(response.getBody(), containsString("{\"message\":\"breakpoint created\",\"breakpointId\":\""));
    }

    @Test
    public void getResponseFromDeleteBreakpointEndpoint() throws Exception {
        Response response = new HttpPetitionClient().sendRequest(new HttpRequest("DELETE", framework.serverState.ServerState.state().getRemoteAddress() + ":8080/simulation32/Breakpoint", "{\"breakpointId\":\"tiempo\"}"));
        assertEquals(200, response.getStatusCode());
        assertEquals("{\"message\":\"Ok breakpoint deleted\"}", response.getBody());
    }

    @Test
    public void getResponseFromRootObjectsEndpointOnSimulation() throws Exception {
        Response response = new HttpPetitionClient().sendRequest(new HttpRequest("GET", "localhost:50000/RootObjects", ""));
        assertEquals(200, response.getStatusCode());
        assertEquals(ROOTS, response.getBody());
    }

    @Test
    public void getResponseFromRootObjectsEndpointOnDispatcher() throws Exception {
        Response response = new HttpPetitionClient().sendRequest(new HttpRequest("GET", "localhost:8080/simulation32/RootObjects", ""));
        assertEquals(200, response.getStatusCode());
        assertEquals(ROOTS, response.getBody());
    }

    @Test
    public void getResponseFromSimulationsEndpointOnDispatcher() throws Exception {
        Response response = new HttpPetitionClient().sendRequest(new HttpRequest("GET", "localhost:8080/simulation32/Simulations", ""));
        assertEquals(200, response.getStatusCode());
        assertThat(response.getBody(), containsString("{\"name\":\"simulation32\"}"));
    }

    @Test
    public void subscribeWebSocketUser() throws Exception {
        new integrationTest.Main.Service("simulation43", integrationTest.Main.initialDate(), integrationTest.Main.finalDate());

        WebSocketClient mock = mock(WebSocketClient.class);
        WebSocketClient webSocketClient = getWebSocketClient(mock);

        webSocketClient.connect();
        Thread.sleep(1000);
        webSocketClient.send("Jose:simulation43");
        Thread.sleep(1000);

        assertTrue(NotificationServer.instance().isSubscribed(new UserIdentification("Jose", "simulation43")));
    }

    @Test
    public void webSocketReceiveMessages() throws Exception {

        new integrationTest.Main.Service("simulation44", integrationTest.Main.initialDate(), integrationTest.Main.finalDate());

        WebSocketClient mock = mock(WebSocketClient.class);
        WebSocketClient webSocketClient = getWebSocketClient(mock);

        webSocketClient.connect();
        Thread.sleep(1000);
        webSocketClient.send("Jose:simulation44");

        ServicesManager.get(NotificationService.class).push("Jose", "{\"message\":\"are you there?\"}");
        verify(mock).onMessage("{\"message\":\"are you there?\"}");
    }

    private WebSocketClient getWebSocketClient(WebSocketClient webClientMocked) throws URISyntaxException {
        return new WebSocketClient(new URI("ws://localhost:8081")) {
            @Override
            public void onOpen(ServerHandshake handshakedata) {
                webClientMocked.onOpen(handshakedata);
            }

            @Override
            public void onMessage(String message) {
                webClientMocked.onMessage(message);
            }

            @Override
            public void onClose(int code, String reason, boolean remote) {
                webClientMocked.onClose(code, reason, remote);
            }

            @Override
            public void onError(Exception ex) {
                webClientMocked.onError(ex);
            }
        };
    }
}
