package sites;

import sites.route.HandlerDictionary;
import sites.route.handler.NotificationHandler;
import sites.route.identification.UserIdentification;
import sites.route.notification.NotificationConnection;
import sites.route.notification.NotificationServer;
import sites.route.notification.WebSocketConnection;
import model.conection.HttpRequest;
import model.conection.Response;
import model.handler.Handler;
import org.java_websocket.WebSocket;
import org.junit.Before;
import org.junit.Test;

import static exception.ExceptionHandler.getSafe;
import static junit.framework.Assert.assertEquals;
import static junit.framework.TestCase.assertSame;
import static org.mockito.Mockito.mock;

public class NotificationTest {

    private NotificationConnection connection;

    @Before
    public void setUp() throws Exception {
        connection = new WebSocketConnection(35000);
        NotificationServer.instance().setConnection(connection);
    }

    @Test
    public void dictionaryFindNotificationsHandler() {
        Handler handler = new HandlerDictionary().find(getSafe(() -> new HttpRequest("POST", "/notification", "{\"sensor\":14}")));
        assertSame(NotificationHandler.class, handler.getClass());
    }

    @Test
    public void sendErrorMessageWhenUserNotExist() {
        connection.newConnection(new UserIdentification("Francisco", "32"), mock(WebSocket.class));
        HttpRequest request = getSafe(() -> new HttpRequest("POST", "/notificiation", "{\"simulationId\":\"32\", \"username\":\"Juan\", \"temperature\":\"15\"}"));
        Response response = new NotificationHandler().handle(request);
        assertEquals(500, response.getStatusCode());
    }

    @Test
    public void receiveANotificationFromSimulation32ToUserFrancisco() {
        connection.newConnection(new UserIdentification("Francisco", "32"), mock(WebSocket.class));
        Response response = new NotificationHandler().handle(getSafe(() -> new HttpRequest("POST", "/notificiation", "{\"simulationId\":\"32\", \"username\":\"Francisco\", \"temperature\":\"32\"}")));
        assertEquals("{\"message\":\"Ok\"}", response.getBody());
        assertEquals(200, response.getStatusCode());
    }

    @Test
    public void sendANotificationToUserJuan() {
        new NotificationHandler().handle(getSafe(() -> new HttpRequest("POST", "/notificiation", "{\"simulationId\":\"32\", \"username\":\"Juan\", \"temperature\":\"15\"}")));
//        verify(connection).send(mockSocket, "{\"simulationId\":\"32\", \"username\":\"Juan\", \"temperature\":\"15\"}");
    }

    private NotificationConnection createConnection() {
        return mock(NotificationConnection.class);
    }

}
